<?php
/**
 * WP Gallery plugin class
 *
 * @package	Gallery WP plug-in
 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
 * @copyright Copyright (c) 2008, Picturesurf.org
 */
class PicturesurfGallery
{
	var $version = "3.0.0";
	var $inited = false;
	var $wpdb;
	var $includesDir;
	var $blogUrl;
	var $pluginUrl;
	var $uploadDir;
	var $galleryDirName;
	var $dataDir;
	var $galleryDir;
	var $dataUrl;
	var $actionsClass;
	var $thumbModes = array('ST' , 'SQ'); //Warning! First should be the grid thumbnail mode!
	var $params;
	var $patchedOneTwo = false;
	var $debug = array('on' => false);
	var $stocks = array();
	/*
	 * Gallery plugin constructor class
	 * 
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function PicturesurfGallery ($actFile)
	{
		$this->includesDir = dirname(__FILE__) . '/';
		require_once ($this->includesDir . 'GalleryActions.php');
		$this->actionsClass = new PicturesurfGalleryActions($this);
		add_action('init', array($this , 'updateRewriter'));
		add_action('admin_menu', array($this , 'addCreatePanel'));
		add_action('edit_post', array($this , 'clearSessionAddMeta'));
		add_action('save_post', array($this , 'clearSessionAddMeta'));
		add_action('publish_post', array($this , 'clearSessionAddMeta'));
		//add_action('wp_dashboard_setup', array($this , 'statsWidgetAdd'));
		if($this->debug['on'])
		{
			$this->debug['started'] = microtime();
			add_action('wp_footer', array($this , 'displayDebugInfo'));
		}
		add_filter('the_posts', array($this , 'replaceWithImage'));
		add_filter('get_the_excerpt', array($this , 'emptyExcerptEmbedGallery'), 0);
		add_filter('the_excerpt_rss', array($this , 'embedGallery'));
		add_filter('the_excerpt', array($this , 'embedGallery'));
		add_filter('the_content', array($this , 'embedGallery'));
		register_activation_hook($actFile, array($this , 'install'));
		register_deactivation_hook($actFile, array($this , 'uninstall'));
		//assigning ajax actions for gallery create
		add_action('wp_ajax_PSaddImage', array($this->actionsClass , 'addImage'));
		add_action('wp_ajax_PSthumbnailImage', array($this->actionsClass , 'thumbnailImage'));
		add_action('wp_ajax_PSsaveSession', array($this->actionsClass , 'saveSession'));
		add_action('wp_ajax_PSsaveDefaults', array($this->actionsClass , 'saveDefaults'));
		add_action('wp_ajax_PSsaveGallery', array($this->actionsClass , 'saveGallery'));
		add_action('wp_ajax_PSregisterImage', array($this->actionsClass , 'registerImage'));
		add_action('wp_ajax_PSloadOriginal', array($this->actionsClass , 'loadOriginal'));
		add_action('wp_ajax_PSdeleteImage', array($this->actionsClass , 'deleteImage'));
		add_action('wp_ajax_PSsaveSettings', array($this->actionsClass , 'saveSettings'));
	}
	/*
	 * display debug info for bugfixing and profiling of the plugin
	 * 
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function displayDebugInfo()
	{
		$timeRun = microtime() - $this->debug['started'];
		echo 'Approx time taken: ' . $timeRun . '<br/>';
		echo 'Total Number of DB Queries: ' . $this->wpdb->num_queries . '<br/>';
		echo 'WP_CACHE: ' . (WP_CACHE===true?'enabled':'disabed') . '<br/>';
		if(WP_CACHE===true)
		{
			global $wp_object_cache;
			echo 'Cache hits: ' . $wp_object_cache->cache_hits . '<br/>';
			echo 'Cache misses: ' . $wp_object_cache->cache_misses . '<br/>';
			echo 'Picturesurf cash: <div>';
			print_r($wp_object_cache->cache['Picturesurf']);
			echo '</div>';
		}
	}
	/*
	 * init variables classes and etc - all of this code was originally in constructor, but 
	 * cause some DB calls are made to get settings this was moved here to be called only when gallery is really needed
	 * 
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function _init()
	{
		if($this->inited) //skip if already called
		{
			return;
		}
		@ini_set('pcre.backtrack_limit', 10000000);
		global $wpdb, $wp_the_query;
		$this->wpdb = &$wpdb;
		require_once ($this->includesDir . 'config.php');
		require_once ($this->includesDir . 'json.php');
		$this->_initParams();
		$this->stocks = array('Picapp', 'Fotoglif');
		$this->params->Settings['OverrideWidth'] = false;
		$this->params->Settings['OverrideDisplay'] = false;
		if(PSG_LOCAL_OVERRIDE_WIDTH)
		{
			$overWidth = PSG_LOCAL_OVERRIDE_WIDTH < 2?2:PSG_LOCAL_OVERRIDE_WIDTH;
			$overWidth = PSG_LOCAL_OVERRIDE_WIDTH >10?10:$overWidth;
			$this->params->Settings['OverrideWidth'] = $overWidth;
		}
		if(PSG_LOCAL_OVERRIDE_DISPLAY_MODE)
		{
			$this->params->Settings['OverrideDisplay'] = true;
			$this->params->Settings['IsImagePages'] = (PSG_LOCAL_OVERRIDE_DISPLAY_MODE == 'ajax');
		}
		if(PSG_LOCAL_STOCKLIMIT)
		{
			$this->stocks = array(PSG_LOCAL_STOCKLIMIT);
			$this->params->Settings['StockImages'] = PSG_LOCAL_STOCKLIMIT;
		}
		$mainDir = str_replace('/includes', '', str_replace('\\', '/', dirname(__FILE__)));
		$this->blogUrl = get_option('siteurl');
		$dir = array_pop(explode('/wp-content/', $mainDir));
		$this->pluginUrl = $this->blogUrl . '/wp-content/' . $dir . '/';
		$this->uploadDir = wp_upload_dir();
		if(!$this->uploadDir['basedir'] && $this->uploadDir['path'] && $this->uploadDir['subdir'])
		{
			$this->uploadDir['basedir'] = str_replace($this->uploadDir['subdir'], '', $this->uploadDir['path']);
		}
		if(!$this->uploadDir['baseurl'] && $this->uploadDir['url'] && $this->uploadDir['subdir'])
		{
			$this->uploadDir['baseurl'] = str_replace($this->uploadDir['subdir'], '', $this->uploadDir['url']);
		}
		$this->galleryDirName = '__galleries/';
		$this->dataDir = str_replace('\\', '/', $this->uploadDir['basedir']) . '/' . $this->params->Settings['SaveUploadsFolder'] . '/';
		$this->galleryDir = $this->dataDir . $this->galleryDirName;
		$this->dataUrl = $this->uploadDir['baseurl'] . '/' . $this->params->Settings['SaveUploadsFolder'] . '/';
		if (! is_dir($this->dataDir))
		{
			@mkdir($this->dataDir);
		}
		if (! is_dir($this->galleryDir))
		{
			@mkdir($this->galleryDir);
		}
		if (! file_exists($this->galleryDir . 'preview.json'))
		{
			@copy($this->includesDir . 'preview.json', $this->galleryDir . 'preview.json');
		}
		$this->inited = true;
		$this->_patch();
	}
	/*
	 * install plugin DB tables and etcs on activation
	 * 
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function install ()
	{
		global $wp_rewrite;
		$this->_init();
		$this->updateRewriter(); //create the rewrite rules 
		$wp_rewrite->flush_rules(); //in MU by default image-pages do not work, until you re-flush rules
		//creating galleries table
		$tblName = $this->wpdb->prefix . "psgalleries";
		if ($this->wpdb->get_var("SHOW TABLES LIKE '$tblName'") != $tblName)
		{
			$sql = 'CREATE TABLE `' . $tblName . '` (
				`id` int(11) NOT NULL auto_increment,
				`code` varchar(100) default NULL,
				`title` varchar(100) default NULL,
				`created` bigint(20) default NULL,
				`views` int(11) default 0,
				PRIMARY KEY  (`id`),
				INDEX (`code`)
			) ENGINE=MyISAM  DEFAULT CHARSET=utf8;';
			$this->wpdb->query($sql);
			//for older MySQLs:
			if ($this->wpdb->get_var("SHOW TABLES LIKE '$tblName'") != $tblName)
			{
				$sql = 'CREATE TABLE `' . $tblName . '` (
					`id` int(11) NOT NULL auto_increment,
					`code` varchar(100) default NULL,
					`title` varchar(100) default NULL,
					`created` bigint(20) default NULL,
					`views` int(11) default 0,
					PRIMARY KEY  (`id`),
					INDEX (`code`)
				) TYPE=MyISAM;';
				$this->wpdb->query($sql);
			}
		}
		//creating imagepages table
		$tblName = $this->wpdb->prefix . "psimagepages";
		if ($this->wpdb->get_var("SHOW TABLES LIKE '$tblName'") != $tblName)
		{
			$sql = 'CREATE TABLE `' . $tblName . '` (
				`id` int(11) NOT NULL,
				`gal_id` int(11) default NULL,
				`sequence` int(6) default 0,
				`title` varchar(100) default NULL,
				`description` text default NULL,
				`image_url` text default NULL,
				`views` int(11) default 0,
				INDEX  (`id`),
				INDEX  (`gal_id`)
			) ENGINE=MyISAM  DEFAULT CHARSET=utf8;';
			$this->wpdb->query($sql);
			//for older MySQLs:
			if ($this->wpdb->get_var("SHOW TABLES LIKE '$tblName'") != $tblName)
			{
				$sql = 'CREATE TABLE `' . $tblName . '` (
					`id` int(11) NOT NULL,
					`gal_id` int(11) default NULL,
					`sequence` int(6) default 0,
					`title` varchar(100) default NULL,
					`description` text default NULL,
					`image_url` text default NULL,
					`views` int(11) default 0,
					INDEX  (`id`),
					INDEX  (`gal_id`)
				) TYPE=MyISAM;';
				$this->wpdb->query($sql);
			}
			$this->_initParams();
			add_option('PSGalleryParameters', serialize($this->params));
			if(WP_CACHE===true)
			{
				wp_cache_set('PSGalleryParameters', serialize($this->params), 'Picturesurf');
			}
			$this->patchedOneTwo = true;
		}
	}
	/*
	 * load and process plugin parameters (version, settings, defaults)
	 * 
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function _initParams()
	{
		if(WP_CACHE===true)
		{
			$parsstr = wp_cache_get('PSGalleryParameters', 'Picturesurf');
			$pars = @unserialize($parsstr);
		}
		if(!$pars || !$pars->Version || !$pars->Settings) //if cache disabled or empty
		{
			$parsstr = get_option('PSGalleryParameters');
			$pars = @unserialize($parsstr);
		}
		if($pars && $pars->Version && $pars->Settings)
		{
			$this->patchedOneTwo = true;
			$this->params = $pars;
			if($parsstr && WP_CACHE===true)
			{
				wp_cache_set('PSGalleryParameters', $parsstr, 'Picturesurf');
			}
		}
		else
		{
			$sets = get_option('GallerySettings'); //v1.2
			$defs = get_option('GalleryDefaults'); //v1.2
			if(!$sets)
			{
				$sets = get_option('PSGallerySettings');//v2.0
				delete_option('PSGallerySettings');
			}
			else
			{
				delete_option('GallerySettings');
			}
			if(!$defs)
			{
				$defs = get_option('PSGalleryDefaults');//v2.0
				delete_option('PSGalleryDefaults');
			}
			else
			{
				delete_option('GalleryDefaults');
			}
			$sets = is_array($sets)?array_pop($sets):$sets;
			$defs = is_array($defs)?array_pop($defs):$defs;
			$sets = $sets?unserialize($sets):array(
				'Width' => PSG_LOCAL_COLS,
				'Height' => PSG_LOCAL_ROWS,
				'IsShowEmail' => PSG_LOCAL_SHOW_EMAIL_SHARE,
				'IsShowFB' => PSG_LOCAL_SHOW_FACEB_SHARE,
				'IsShowPL' => PSG_LOCAL_SHOW_PERML_SHARE,
				'IsPages' => PSG_LOCAL_MODE=='page',
				'IsAjax' => PSG_LOCAL_MODE=='ajax',
				'IsInline' => PSG_LOCAL_MODE=='inline',
				'StockImages' => PSG_LOCAL_STOCKIMAGES,
				'IsSavedOnce' => false,
				'SaveUploadsFolder' => PSG_LOCAL_SAVE_IMG_FOLDER);
			//combine Version, Defaults and Settings into 1 WP option
			$obj = new stdClass();
			$obj->Version = $this->version;
			$obj->Settings = $sets;
			$obj->Defaults = $defs;
			$this->params = $obj;
		}
	}
	/*
	 * patch previous version of plugin, or data left from it to current one
	 * 
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function _patch ()
	{
		$TblImages = $this->wpdb->prefix . "psimagepages";
		$TblGalleries = $this->wpdb->prefix . "psgalleries";
		if($this->wpdb->get_var("SHOW TABLES LIKE '$TblGalleries'") != $TblGalleries || $this->wpdb->get_var("SHOW TABLES LIKE '$TblImages'") != $TblImages)
		{
			$this->wpdb->query('DROP TABLE ' . $TblGalleries . ';');
			$this->wpdb->query('DROP TABLE ' . $TblImages . ';');
			$this->install();
			return;
		}
		//plugin version to version updates and fixes
		if($this->version != $this->params->Version) //in case of 1.2 patching versions do match
		{
			$V = $this->_versionNumber($this->params->Version);
			if($V < 20005) //changes introduced in 2.0.5
			{
				$this->wpdb->query('ALTER TABLE ' . $TblGalleries . ' ADD INDEX (code);');
				$this->wpdb->query('ALTER TABLE ' . $TblImages . ' MODIFY gal_id int(11) default NULL, MODIFY sequence int(6) default 0;');
				$this->params->Version = $this->version;
				update_option('PSGalleryParameters', serialize($this->params));
			}
			$V = $this->_versionNumber($this->params->Version);
			if($V < 30000) //changes introduced in 3.0
			{
				$this->params->Settings = array( //update settings object
					'Width' => PSG_LOCAL_COLS,
					'Height' => PSG_LOCAL_ROWS,
					'IsShowEmail' => $this->params->Settings['IsShowEmail'],
					'IsShowFB' => $this->params->Settings['IsShowFaceB'],
					'IsShowPL' => PSG_LOCAL_SHOW_PERML_SHARE,
					'IsPages' => $this->params->Settings['IsImagePages'],
					'IsAjax' => !$this->params->Settings['IsImagePages'],
					'IsInline' => false,
					'StockImages' => PSG_LOCAL_STOCKIMAGES,
					'IsSavedOnce' => false,
					'SaveUploadsFolder' => PSG_LOCAL_SAVE_IMG_FOLDER);
				$this->params->Version = $this->version;
				unset($this->params->Defaults);
				update_option('PSGalleryParameters', serialize($this->params));
			}
		}
		if($this->patchedOneTwo)
		{
			return;
		}
		//huge 1.2 into  2.0 patch
		$oldVer = get_option("GalleryVersion");
		$newVer = get_option("PSGalleryVersion");
		if(!$oldVer && !$newVer) 
		{
			$oldVer = '1.2'; //set 1.2 as default so that patching applies, even if version is lost
		}
		if ($oldVer == '1.0') //Deprecated, we shouldn't have any user having this one
		{
			$this->wpdb->query('DROP TABLE ' . $this->wpdb->prefix . 'psgalleries;');
		}
		if ($oldVer == '1.2')
		{
			//check patch lock
			$patchinglock = $this->dataDir . 'patchinglocked.txt';
			if(file_exists($patchinglock))
			{
				return; //quit patching
			}
			@error_log("\r\n====== PATCHING STARTED " . date("M j Y H:i:s") . "\r\n\r\n", 3, $this->dataDir.'patchlog.txt');
			if (! is_dir($this->dataDir) || !is_dir($this->galleryDir) || !file_exists($this->galleryDir . 'preview.json'))
			{
				@error_log("\r\n### Error - patch failed new data folders do not exist\r\n", 3, $this->dataDir . 'patchlog.txt');
				return; //exit patching if data folders do not exist
			}
			$fp = fopen($patchinglock, 'w'); //Place the patching lock
			fclose($fp);
			$this->_extractImagesAndDataFiles($this->dataDir, true);
			$oldDataDir = str_replace('/includes', '', str_replace('\\', '/', dirname(__FILE__))) . '/data/';
			if (is_dir($oldDataDir)) //if previous plugin's data folder exists
			{
				$this->_extractImagesAndDataFiles($oldDataDir);
			}
			if(is_dir($this->dataDir . '_backup')) //grab all json files from backup and move to galleries folder
			{
				$dir = opendir($this->dataDir . '_backup');
				while ($file = @readdir($dir))
				{
					if($file != '..' && $file != '.')
					{
						rename($this->dataDir . '_backup' . $file, $this->galleryDir . $file);
					}
				}
				if(!@rmdir($this->dataDir . '_backup'))
				{
					@error_log("\r\n### Error - patch failed _backup folder can't be deleted\r\n", 3, $this->dataDir.'patchlog.txt');
					return; //exit patching if delete problem
				}
			}
			$json = new PSGalleryServicesJSON();
			delete_option("GalleryVersion");
			delete_option("PicturesurfAPIKey");
			ob_start();
			$this->wpdb->show_errors();
			$records = $this->wpdb->get_results('SELECT * FROM `' . $TblGalleries . '`;');
			foreach($records as $record)
			{
				$TblPMeta = $this->wpdb->prefix . 'postmeta';
				$this->wpdb->query('UPDATE ' . $TblPMeta . ' SET meta_value = ' . $record->id . ' WHERE meta_key="_Gallery" AND meta_value="'. $record->ps_id .'";');
			}
			$this->wpdb->query('ALTER TABLE ' . $TblImages . ' CHANGE COLUMN ps_id id INT(11);');
			$this->wpdb->query('ALTER TABLE ' . $TblImages . ' CHANGE COLUMN preview_url image_url text default NULL;');
			$this->wpdb->query('ALTER TABLE ' . $TblImages . ' ADD COLUMN gal_id INT(11) default NULL AFTER id;');
			$this->wpdb->query('ALTER TABLE ' . $TblImages . ' MODIFY sequence int(6) default 0;');			
			$this->wpdb->query('ALTER TABLE ' . $TblGalleries . ' ADD COLUMN title varchar(100) default NULL;');
			$this->wpdb->query('ALTER TABLE ' . $TblGalleries . ' ADD COLUMN created BIGINT(20) default NULL;');
			$this->wpdb->query('ALTER TABLE ' . $TblGalleries . ' ADD COLUMN views BIGINT(20) DEFAULT 0;');
			$this->wpdb->query('ALTER TABLE ' . $TblGalleries . ' DROP COLUMN ps_id;');
			$this->wpdb->query('ALTER TABLE ' . $TblGalleries . ' ADD INDEX (code);');
			$dberrors = ob_get_clean();
			if($dberrors)
			{
				@error_log("\r\n### Error - patch failed due to DB errors :\r\n" . $dberrors, 3, $this->dataDir.'patchlog.txt');
				echo $dberrors;
				return; //exit if any DB quiry problem
			}
			//run through all saved galleries and update them
			$this->wpdb->query('TRUNCATE TABLE `' . $TblImages . '`;');
			$created = time();
			foreach($records as $record)
			{
				$data = @file_get_contents($this->galleryDir . $record->code . '.json');
				if(!$data)
				{
					$data = @file_get_contents(PICTURESURF_SERVER . '/gallery_data/data/' . $record->code . '.json');
					if($data)
					{
						$filepath = $this->galleryDir . $record->code . '.json';
						$fp = @fopen($filepath, "w");
						fwrite($fp, $data);
						fclose($fp);
						$data = @file_get_contents($this->galleryDir . $record->code . '.json');
					}
				}
				if(!$data)
				{
					@error_log("\r\n### Error - No data found for gallery " . $record->code . ":\r\n", 3, $this->dataDir.'patchlog.txt');
					$this->_cleanGalleryFromPosts($record->code);
					continue;
				}
				$convdata = stripcslashes(rawurldecode(str_replace("Gallery = ", "", $data)));
				$gal = $json->decode($convdata);
				if(!$gal || !$gal->gid) //if corrupted gata - erase the gallery
				{
					@error_log("\r\n### Error - Can't restore data for gallery " . $record->code . ":\r\n" . $convdata, 3, $this->dataDir.'patchlog.txt');
					$this->_cleanGalleryFromPosts($record->code);
					continue;
				}
				$gal->gid = $record->id;
				$gal->code = $record->code;
				$newDir = $this->actionsClass->_getImageSavePath($gal->title, $gal->gid);
				$i = 1;
				foreach ($gal->ims as $key => $img) //update images locations
				{
					foreach ($img->thumbnails as $mode => $thumb)
					{
						$name = array_pop(explode('/', $thumb));
						if(file_exists($this->dataDir . $name) && @rename($this->dataDir . $name, $newDir['path'] . '/' . $name))
						{
							$img->thumbnails->$mode = $this->dataUrl . $newDir['name'] . '/' . $name;
						}
					}
					$img->src = $img->thumbnails->orig;
					$query = 'INSERT INTO `' . $TblImages . '` (id, gal_id, sequence, title, description, image_url, views) ' . 
					' VALUES (%d, %d, %d, %s, %s, %s, 0);';
					$this->wpdb->query($this->wpdb->prepare($query, $img->id, $record->id, $i, $img->title, $img->descr, $img->src));
					$gal->ims[$key] = $img;
					$i++;
				}
				$this->wpdb->query($this->wpdb->prepare('UPDATE `' . $TblGalleries . '` SET title = %s, created = %d;', $gal->title, $created));
				$datastr = "Gallery = " . $json->encode($gal);
				$data_filepath = $this->galleryDir . $gal->code . '.json';
				$fp = @fopen($data_filepath, "wb");
				fwrite($fp, $datastr);
				fclose($fp);
			}
			delete_option('GalleryDataString');
			@error_log("\r\n====== PATCHING ENDED " . date("M j Y H:i:s") . "\r\n\r\n", 3, $this->dataDir.'patchlog.txt');
			unlink($patchinglock);
		}
		delete_option('PSGalleryVersion');
		delete_option('PSGalleryDataString');
		add_option('PSGalleryParameters', serialize($this->params));
		if(WP_CACHE===true)
		{
			wp_cache_set('PSGalleryParameters', serialize($this->params), 'Picturesurf');
		}
	}
	/*
	 * grab all images from previous data folder and move them into root of the new one
	 * 
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function _extractImagesAndDataFiles($oldFolder, $clear = false)
	{
		$dir = opendir($oldFolder);
		while ($file = readdir($dir)) //move all contents of data folder into new data folder
		{
			if($file != '..' && $file != '.')
			{
				@error_log("\r\n### *Processing data folder item: " . $file , 3, $this->dataDir.'patchlog.txt');
				if(is_dir($oldFolder . $file) && $file != '_backup' && $file != $this->galleryDirName) //if it's an images folder
				{
					$sdir = opendir($oldFolder . $file);
					while ($sfile = readdir($sdir)) //grab all images and put them new root folder
					{
						if($sfile != '..' && $sfile != '.')
						{
							$fPath = $oldFolder . $file . '/' . $sfile;
							if(getimagesize($fPath))
							{
								$clear?rename($fPath, $this->dataDir . $sfile):copy($fPath, $this->dataDir . $sfile);
							}
						}
					}
					if($clear)
					{
						@rmdir($oldFolder . $file);
					}
				}
				elseif($file == '_backup')
				{
					$sdir = opendir($oldFolder . $file);
					while ($sfile = readdir($sdir)) //grab all data files and put them in new galleries folder
					{
						if($sfile != '..' && $sfile != '.')
						{
							$fPath = $oldFolder . $file . '/' . $sfile;
							$clear?rename($fPath, $this->dataDir . $sfile):copy($fPath, $this->galleryDir . $sfile);
						}
					}
				}
				elseif(!file_exists($this->dataDir . $file)) //move all folder files
				{
					$clear?rename($oldFolder . $file, $this->dataDir . $file):copy($oldFolder . $file, $this->dataDir . $file);
				}
			}
		}
	}
	/*
	 * verify that permalink setting doesn't have unsafe charecters
	 * 
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function _verifyPermalinkStructure()
	{
		global $wp_rewrite;
		if($wp_rewrite->permalink_structure)
		{
			$matches = array();
			preg_match('@[\ \+\?]@', $wp_rewrite->permalink_structure, $matches);
			if(count($matches) > 0)
			{
				$message = '<strong>Oops!</strong> Your gallery will not function properly because your Permalinks have characters such as:';
				$message.= '<p style="border:1px solid #CCCCCC; background-color: #FFFFFF; padding:5px"><strong>+ ?</strong> and space (" ")</p>';
				$message.= '<div>To fix your gallery go into your Settings under the Customize Permalink Structure section and remove these characters from your Custom Structure.</div>';
				$this->error($message);
			}
		}
	}
	/*
	 * verify all of needed folders exist and have correct permissions
	 * 
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function _verifyFolders()
	{
		if (!$this->uploadDir['basedir'] || !$this->uploadDir['baseurl'])
		{
			if($this->uploadDir['error'])
			{
				$message = '<strong>Oops!</strong> Our plugin cannot upload files because Wordpress have a problem working with your uploads folder.';
				$message .= '<div style="font-weight:bold; margin-top:5px;">Please examine the message Wordpress reports and ask Wordpress team for support if you can\'t figure this out on your own.</div>';
				$message .= '<p style="border:1px solid #CCCCCC; background-color: #FFFFFF; padding:5px">' . $this->uploadDir['error'] . '</p>';
			}
			else
			{
				$message = '<strong>Oops!</strong> Our plugin cannot find the folder that holds your uploaded files.';
				$message .= '<div style="font-weight:bold; margin-top:5px;">Please verify that this folder exists and has write access:</div>';
				$folder = '<font style="font-style:italic">your installation path</font>/wp-content/uploads';
				$message .= '<p style="border:1px solid #CCCCCC; background-color: #FFFFFF; padding:5px">' . $folder . '</p>';
			}
			$this->error($message);
			return false;
		}
		if (! is_dir($this->dataDir))
		{
			$message = '<strong>Oops!</strong> Our plugin cannot find the folder that holds your gallery images.';
			$message .= '<div style="font-weight:bold; margin-top:5px;">Please verify that this folder exists:</div>';
			$message .= '<p style="border:1px solid #CCCCCC; background-color: #FFFFFF; padding:5px">' . $this->dataDir . '</p>';
			$this->error($message);
			return false;
		}
		$testfile = $this->dataDir . 'test1.txt';
		$fp = @fopen($testfile, 'w');
		if (! $fp)
		{
			$message = '<strong>Oops!</strong> A permission setting is preventing our plugin from putting images into the gallery folder.';
			$message .= '<div style="font-weight:bold; margin-top:5px;">Please enable write access for this folder:</div>';
			$message .= '<p style="border:1px solid #CCCCCC; background-color: #FFFFFF; padding:5px">' . $this->dataDir . '</p>';
			$message .= '<strong>...or let your server administrator resolve this issue by sending them the following message:</strong>';
			$mail = 'Hello,<br/><br/>' . 'Please enable write access for the folder:<br/>' . $this->dataDir . '<br/><br/>' . 
				'I need this setting changed in order to run a WordPress plugin for creating image galleries.<br/><br/>' . 'Thanks!';
			$this->error($message, $mail);
			return false;
		}
		fclose($fp);
		@unlink($testfile);
		if (! is_dir($this->galleryDir))
		{
			if (! @mkdir($this->galleryDir, 0777))
			{
				$message = '<strong>Oops!</strong> Our plugin cannot find the folder that holds backups of your gallery data.';
				$message .= '<div style="font-weight:bold; margin-top:5px;">Please verify that this folder exists:</div>';
				$message .= '<p style="border:1px solid #CCCCCC; background-color: #FFFFFF; padding:5px">' . $this->galleryDir . '</p>';
				$this->error($message);
				return false;
			}
		}
		$testfile = $this->galleryDir . 'test2.txt';
		$fp = @fopen($testfile, 'w');
		if (! $fp)
		{
			$message = '<strong>Oops!</strong> A permission setting is preventing our plugin from backing up your gallery data.';
			$message .= '<div style="font-weight:bold; margin-top:5px;">Please enable write access for this folder:</div>';
			$message .= '<p style="border:1px solid #CCCCCC; background-color: #FFFFFF; padding:5px">' . $this->galleryDir . '</p>';
			$message .= '<strong>...or let your server administrator resolve this issue by sending them the following message:</strong>';
			$mail = 'Hello,<br/><br/>' . 'Please enable write access for the folder:<br/>' . $this->galleryDir . '<br/><br/>' . 
				'I need this setting changed in order to run a WordPress plugin for creating image galleries.<br/><br/>' . 'Thanks!';
			$this->error($message, $mail);
			return false;
		}
		fclose($fp);
		@unlink($testfile);
		return true;
	}
	/*
	 * remove gallery code from the post text in WP DB
	 * 
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function _cleanGalleryFromPosts($code)
	{
		$this->wpdb->query('DELETE FROM ' . $this->wpdb->prefix . 'psgalleries WHERE code = "' . $code . '";');
		$pat = '[PSGallery=' . $code . ']';
		$this->wpdb->show_errors();
		$posts = $this->wpdb->get_results('SELECT ID, post_content FROM ' . $this->wpdb->prefix . 'posts WHERE post_content LIKE "%' . $pat . '%";');
		foreach($posts as $post)
		{
			$cont = str_replace($pat, '', $post->post_content);
			$this->wpdb->query($this->wpdb->prepair('UPDATE ' . $this->wpdb->prefix . 'posts SET post_content = %s WHERE ID = ' . $post->ID, $cont));
		}
	}
	/*
	 * show initialization error message in formatted div
	 * 
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function error ($message, $mail = null)
	{
		echo '<div style="width:656px; margin:10px auto; font-size: 13px; padding:17px; border:1px solid #DD3C10; background-color: #FFEBE8">' . 
			'<div>' . $message . '</div>' . ($mail ? '<p style="border:1px solid #CCCCCC; background-color: #FFFFFF; padding:7px">' . $mail . '</p>' : '') . 
			'<div>Still need help? <a href="http://getsatisfaction.com/picturesurf/products/picturesurf_picturesurf_gallery_plugin_for_wordpressorg"' . 
			' target="_blank">Post on our support forum</a> or ' . 
			'<a href="mailto:contact@picturesurf.org">contact us directly</a> so we can help you resolve the issue quickly ;)</div>' . '</div>';
	}
	/*
	 * clean up plugin DB tables and etcs on deactivation
	 * 
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function uninstall ()
	{
		//TODO should we drop psgalleries table? Galleries will be lost
	}
	/*
	 * Add image-page into url parameters
	 * 
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function updateRewriter ()
	{
		global $wp_rewrite;
		$wp_rewrite->add_endpoint('image-page', EP_ALL);
		$wp_rewrite->rewrite_rules();
	}
	/*
	 * add Gallery panel into WP admin action hooks
	 * 
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function addCreatePanel ($a)
	{
		if (function_exists('add_meta_box')) // 2.5 style
		{
			$head = 'Picturesurf Gallery: <span style="font-weight:normal">Add an image gallery to your blog post in seconds</span>';
			add_meta_box('gallerydiv', $head, array($this , 'drawCreatePanel'), 'post', 'normal');
			add_meta_box('gallerydiv', $head, array($this , 'drawCreatePanel'), 'page', 'normal');
			add_action('admin_head', array($this, 'addCreatePanelCSS'));
		}
	}
	/*
	 * add picturesurf gallery CSS stylesheet into the head tag
	 * 
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function addCreatePanelCSS ()
	{
		$this->_init();
		echo '<link media="all" type="text/css" rel="stylesheet" href="' . $this->pluginUrl . 'css/psgallery.css?'. $this->version . '"/>';		
	}
	/*
	 * check if there is an unlinked gallery inside the post and link it to post meta 
	 * most common cause - WP's autosave do not trigger post_edit action
	 * 
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function _restoreGalleryInDraft($post)
	{
		$matches = array();
		preg_match_all('@\[PSGallery=(\w+)\]@', $post->post_content, $matches, PREG_PATTERN_ORDER);
		if($matches[1])
		{
			$gals = $matches[1];
			foreach ($gals as $galCode)
			{
				$q1 = $this->wpdb->prepare('SELECT id, code FROM ' . $this->wpdb->prefix . 'psgalleries WHERE code=%s', $galCode);
				$res1 = $this->wpdb->get_row($q1);
				if($res1->id)
				{
					$q2 = $this->wpdb->prepare('SELECT post_id FROM ' . $this->wpdb->prefix . 'postmeta WHERE meta_key="_Gallery" AND meta_value=%d', $res1->id);
					$res2 = $this->wpdb->get_row($q2);
					if(!$res2)
					{
						add_post_meta($post->ID, '_Gallery', $res1->id, true);
						return;
					}
				}
			}
		}
	}
	/*
	 * draw preview block, draw create iframe, check for existing post's Gallery, 
	 * load it's data or clear session of previous post
	 * 
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function drawCreatePanel ()
	{
		$this->_init();
		if(!$this->_verifyFolders())
		{
			return;
		}
		$this->_verifyPermalinkStructure();
		global $post, $user_ID;
		get_currentuserinfo();
		if(!$user_ID)
		{
			$message = '<strong>Oops!</strong> We do not see the user logged in!';
			$message .= '<div style="font-weight:bold; margin-top:5px;">If you are not a hacker and you are able to see the edit post page, email the message below to the Wordpress team.</div>';
			$email = 'Hi guys, for some reason get_currentuserinfo() function provide empty result when in fact WP does treat the person as logged in user and displays the post edit page successfully.';
			$email.= ' Please email your thoughts to us: contact@picturesurf.org.</br>Picturesurf Gallery for Worpress team.';
			$message .= '<p style="border:1px solid #CCCCCC; background-color: #FFFFFF; padding:5px">' . $email . '</p>';
			$this->error($message);
			return;
		}
		$json = new PSGalleryServicesJSON();
		$datastr = get_usermeta($user_ID, "PSGalleryDataString");
		$settingsJson = $json->encode($this->params->Settings);
		$datastrCut = stripcslashes(str_replace("Gallery = ", "", $datastr));
		$gal = $json->decode($datastrCut);
		$gidS = ($gal && $gal->gid)?$gal->gid:null;
		if ($post && $post->ID) //if post was previously saved -> check for gallery
		{
			$meta = get_post_meta($post->ID, '_Gallery');
			if(!$meta) //if not meta with gallery id -> try searching for the code
			{
				$this->_restoreGalleryInDraft($post);
				$meta = get_post_meta($post->ID, '_Gallery');
			}
			if(is_array($meta) && count($meta) > 0) //if meta has a gallery -> load it
			{
				$gidP = array_pop($meta);
				if ($gidP != $gidS) //if gallery is not yet in session -> load it
				{
					$row = $this->wpdb->get_row("SELECT code FROM " . $this->wpdb->prefix . "psgalleries WHERE id = '" . $gidP . "';");
					$data_filename = $row->code . '.json';
					$data = @file_get_contents($this->galleryDir . $data_filename); //avoid warning if no file
					$datastr = ($data ? $data : '');
					update_usermeta($user_ID, "PSGalleryDataString", $datastr);
				}
			}
			else if($gidS > 0) //if post do not have gallery linked to it, and in session we have a "saved" gallery -> restart with new
			{
				$datastr = '';
				update_usermeta($user_ID, "PSGalleryDataString", $datastr);
			}
		} 
		else if ($gidS > 0) //if this is a new post -> create new gallery
		{
			$row = $this->wpdb->get_row('SELECT post_id FROM ' . $this->wpdb->prefix . "postmeta WHERE meta_key = '_Gallery' AND meta_value = '" . $gidS . "';");
			if ($row && $row->post_id > 0)
			{
				$datastr = '';
				update_usermeta($user_ID, "PSGalleryDataString", $datastr);
			}
		}
		include_once $this->includesDir . 'gallery_create.php';
	}
	/*
	 * clear Gallery session and add record in post meta when user saves the post where Gallery is inserted
	 * 
	 * @param $newId
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function clearSessionAddMeta ($newId)
	{
		$this->_init();
		global $user_ID;
		get_currentuserinfo();
		$json = new PSGalleryServicesJSON();
		$datastr = stripcslashes(str_replace('Gallery = ', '', get_usermeta($user_ID, 'PSGalleryDataString')));
		$gal = $json->decode($datastr);
		global $post;
		if(function_exists(wp_is_post_revision))
		{
			$parent = wp_is_post_revision($newId);
			$newId = $parent->ID;
		}
		$mainId = $_POST['ID'] ? $_POST['ID'] : $post->ID;
		if ($gal && $gal->gid && $mainId)
		{
			add_post_meta($mainId, '_Gallery', $gal->gid, true) ? '' : update_post_meta($mainId, '_Gallery', $gal->gid);
			if ($newId)
			{
				add_post_meta($newId, '_Gallery', $gal->gid, true) ? '' : update_post_meta($newId, '_Gallery', $gal->gid);
			}
			update_usermeta($user_ID, 'PSGalleryDataString', '');
		}
	}
	/*
	 * replace the WP's original get_the_content filter that cuts html tags out 
	 * to excape and keep the gallery displayed correctly
	 * 
	 * @param $content
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function emptyExcerptEmbedGallery($text)
	{
		if($text == '')
		{
			remove_filter('the_content', array($this , 'embedGallery'));
			$text = get_the_content('');
			$text = strip_shortcodes( $text );
			$text = apply_filters('the_content', $text);
			$text = str_replace(']]>', ']]&gt;', $text);
			$text = strip_tags($text);
			$excerpt_length = apply_filters('excerpt_length', 55);
			$words = explode(' ', $text, $excerpt_length + 1);
			if (count($words) > $excerpt_length) 
			{
				array_pop($words);
				array_push($words, '[...]');
				$text = implode(' ', $words);
			}
			add_filter('the_content', array($this , 'embedGallery'));
		}
		return $text;
	}
	/*
	 * find [myGallery/Gallery=id] embed codes and substitute those with real widget embed
	 * 
	 * @param $content
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function embedGallery ($content)
	{
		global $wp_the_query, $post;
		$matches = array();
		preg_match_all('@\[PSGallery=([^\]]+)\]?@', $content, $matches, PREG_SET_ORDER);
		$patterns = array();
		$replacements = array();
		if(count($matches) > 0) //init only if gallery is inside
		{
			$this->_init();
		}
		foreach ($matches as $Gallery)
		{
			list ($tag, $code) = $Gallery;
			if (isset($code))
			{
				$embed = $this->_getGalleryEmbed($code, $wp_the_query->is_feed, $post->post_status);
				if (isset($embed))
				{
					$patterns[] = $tag;
					$replacements[] = $embed;
				}
			}
		}
		$content = str_replace($patterns, $replacements, $content);
		return $content;
	}
	/*
	 * output embed code for Galleries that reside on Picturesurf.org
	 * 
	 * @param $code
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function _getGalleryEmbed ($code, $is_feed = false, $status = null)
	{
		$json = new PSGalleryServicesJSON();
		$datastr = @file_get_contents($this->galleryDir . $code . '.json');
		if(!$datastr)
		{
			return false;
		}
		if ($is_feed)
		{
			return $this->_getGalleryInHTML($datastr);
		}
		$post_id = (WP_CACHE===true?wp_cache_get('PSPostForGallery' . $code, 'Picturesurf'):null);
		if(!$post_id)
		{
			$row = $this->wpdb->get_row('SELECT post_id FROM ' . $this->wpdb->prefix . 'postmeta AS PM' .
			' LEFT JOIN ' . $this->wpdb->prefix . 'psgalleries AS PSG ON PM.meta_value = PSG.id' . 
			' WHERE PM.meta_key="_Gallery" AND PSG.code="' . $code . '";');
			$post_id = $row->post_id;
			WP_CACHE===true?wp_cache_set('PSPostForGallery' . $code, $post_id, 'Picturesurf'):'';
		}
		if ($post_id > 0 && (!$status || $status!='draft'))
		{
			global $wp_rewrite;
			$postUrl = rawurldecode(get_permalink($row->post_id));
			$prefSlash = (substr($postUrl, - 1) != '/' ? '/' : '');
			$pageAdd = $wp_rewrite->permalink_structure ? $prefSlash . 'image-page/' : (strstr($postUrl, '?') ? '&' : '?') . 'image-page=';
			$html = '<script type="text/javascript">var G_PS_GalleryPagePreurl = "' . $postUrl . $pageAdd . '";</script>';
		}
		//$this->wpdb->query('UPDATE ' . $this->wpdb->prefix . 'psgalleries SET views = views+1 WHERE code = "' . $code . '"');
		$widgSrc = $this->pluginUrl . 'widget/gallery.php?gallery=' . $code . '&dir=' . rawurlencode($this->galleryDir);
		$html .= '<script type="text/javascript">var G_PS_GallerySettings = ' . $json->encode($this->params->Settings) . '; var G_GalSpecialV = "2.0";</script>';
		$html .= '<script type="text/javascript" src="' . $widgSrc . '"></script>';
		$html .= '<noscript>' . $this->_getGalleryInHTML($datastr) . '</noscript>';
		$linkt = 'Blog Gallery';
		$post= 'by <a href="http://www.picturesurf.org">Picturesurf</a>';
		$host = $_SERVER['HTTP_HOST'];
		if(strstr($host, 'techcrunch.com') || strstr($host, 'crunchgear.com') || strstr($host, 'mobilecrunch.com'))
		{
			$linkt = 'Photo Gallery';
			$post= 'by Picturesurf';
		}
		$lnkhtml = '<a title="' . $linkt . '" href="http://www.picturesurf.org/gallery/">' . $linkt . '</a> ';
		$html .= '<p style="padding:0 0 0 4px;margin:0;text-align:left;font-size:0.9em">' . $lnkhtml . $post . '</p>';
		return $html;
	}
	/*
	 * if this is an image post - adds return back to post link
	 * 
	 * @param $content
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function replaceWithImage ($posts)
	{
		global $wp_query;
		if (! is_array($wp_query->query))
		{
			return $posts;
		}
		$imgNum = intval($wp_query->query['image-page']);
		if (! $imgNum) //skip if not image page
		{
			$url = $_SERVER['REQUEST_URI'];
			if (strstr($url, 'image-page'))
			{
				$redirect = preg_replace('@image-page[/]@', '', $url);
				header('Location: ' . $redirect);
				exit();
			}
			return $posts;
		}
		if (count($posts) != 1) //skip if not single post
		{
			return $posts;
		}
		$post = $posts[0];
		$galId = array_pop(array_unique(get_post_meta($post->ID, '_Gallery')));
		if (! $galId)
		{
			return;
		}
		$this->_init();
		$res = $this->wpdb->get_results('SELECT *, IP.id as img_id FROM ' . $this->wpdb->prefix . 'psimagepages AS IP
			LEFT JOIN ' . $this->wpdb->prefix . 'psgalleries AS G ON G.id = IP.gal_id
			WHERE IP.gal_id = "' . $galId . '" ORDER BY IP.sequence;', ARRAY_A);
		if (! $res || ! count($res))
		{
			return;
		}
		global $wp_rewrite;
		$data = array();
		$data['post_url'] = get_permalink($post->ID);
		$data['post_url'].= (($wp_rewrite->permalink_structure && (substr($data['post_url'], - 1) != '/')) ? '/' : '');
		$imageUrlSuf = ($wp_rewrite->permalink_structure ? 'image-page/' : (strstr($data['post_url'], '?') ? '&' : '?') . 'image-page=');
		$imgFound = false;
		$isNext = false;
		$isPrev = false;
		$NextUrl = '#';
		$PrevUrl = '#';
		$ImageClickUrl = '#';
		$IsEmail = $this->params->Settings['IsShowEmail'];
		$IsFaceB = $this->params->Settings['IsShowFB'];
		foreach ($res as $row)
		{
			if ($row['sequence'] == $imgNum - 1)
			{
				$isPrev = true;
				$PrevUrl = $data['post_url'] . $imageUrlSuf . $row['sequence'];
			}
			if ($row['sequence'] == ($imgNum + 1))
			{
				$isNext = true;
				$NextUrl = $data['post_url'] . $imageUrlSuf . $row['sequence'];
				$ImageClickUrl = $data['post_url'] . $imageUrlSuf . $row['sequence'];
			}
			if ($row['sequence'] == $imgNum)
			{
				$data = array_merge($data, $row);
				$imageId = $row['img_id'];
				$imgFound = true;
			}
		}
		if ($isPrev && ! $isNext)
		{
			$ImageClickUrl = $data['post_url'] . $imageUrlSuf . '1';
		}
		if (! $imgFound)
		{
			return;
		}
		/*
		else 
		{
			$this->wpdb->query('UPDATE ' . $this->wpdb->prefix . 'psimagepages SET views = views+1 WHERE id = "' . $imageId . '"');
		}
		*/
		$json = new PSGalleryServicesJSON();
		$datastr = @file_get_contents($this->galleryDir . $data['code'] . '.json');
		if(!$datastr)
		{
			return;
		}
		$gallery = $json->decode(stripcslashes(str_replace('Gallery = ', '', $datastr)));
		$image = null;
		foreach ($gallery->ims as $itm)
		{
			if($itm->id == $data['img_id'])
				$image = $itm;
		}
		if(!$image)
		{
			return;
		}
		$image->title = rawurldecode($image->title);
		$data['description'] = rawurldecode($image->descr);
		if($image->StockMode)
		{
			$w = array_pop(explode('x', $gallery->data->thar)) * 84 - 5;
			if($image->StockMode == 'Picapp')
			{
				$stock = 'data-stock="' . $image->StockMode . '"';
				$tip = htmlentities($image->title) . ' Content &copy; 2008 ' . htmlentities($image->StockAdd);
				$data['image_url'] = $image->thumbnails->original;
				$data['image_object'] = '<img ' . $stock . ' title="' . $tip . '" src="' . $data['image_url'] . '"/>';
				$data['image_ads'] = '<div style="text-align:right; padding:4px 1px 0 0">No endorsement implied</div>';
				$adsUrl = 'http://zen.picapp.com/ads/psads/default.aspx?AdSize=' . ($w <= 468?1:2) . '&imageId=' . $image->StockParam;
				$data['image_ads'].= '<div style="text-align:center; margin-top:15px;" align="center">';
				$data['image_ads'].= '<iframe frameborder="0"" width="' . (($w <= 468?234:468) + 3) . '" height="68" src="' . $adsUrl . '"></iframe></div>';
				//$data['copyright'] = '&copy; <a href="http://www.picapp.com/" rel="nofollow">PicApp</a>';
			}
			if($image->StockMode == 'Fotoglif')
			{
				$size = 'large';
				if($w <= 468*1.3)
				{
					$size = 'medium';
				}
				if($w <= 234*1.3)
				{
					$size = 'small';
				}
				$src = 'http://www.fotoglif.com/embed/embed.py?size=' . $size . '&imageuid=' . $image->StockParam . '&layout=l&jpgembed=yes';
				$data['image_object'] = '<div><div id="fotoglif_place_holder_' . $image->StockParam . '"></div>';
				$data['image_ads'] = '<script type="text/javascript" src="' . $src . '"></script></div>';
				$data['image_url'] = $src;
				//$data['copyright'] = '&copy; <a href="http://www.fotoglif.com/" rel="nofollow">fOTOGLIF</a>';
			}
		}
		else
		{
			$data['image_url'] = $image->thumbnails->original;
			$data['image_object'] = '<img title="' . htmlentities($image->title) . '" src="' . $image->thumbnails->custom . '"/>';
			list ($data['width'], $data['height']) = @getimagesize($image->thumbnails->custom);
			list ($data['orig_w'], $data['orig_h']) = @getimagesize($data['image_url']);
		}
		$data['title'] = $image->title;
		$data['total'] = count($res);
		$data['self_url'] = $data['post_url'] . $imageUrlSuf . $data['sequence'];
		$data['token'] = array_shift(explode('.', $data['image_url']));
		$divId = $data['code'] . rand(001,999);
		$dataFilePath = $this->galleryDir . $data['code'] . '.json';
		ob_start();
		include ($this->includesDir . 'gallery_image.php');
		$post->post_content = preg_replace("@[\r\n\t]@s", "", ob_get_clean());
		$post->custom_css_href = $this->pluginUrl . 'css/' . 'imagepage-css.php?divid=' . $divId . '&path=' . rawurlencode($dataFilePath);
		$posts[0] = $post;
		add_action('get_header', array($this , '_addImagePageCSS'));
		return $posts;
	}
	/*
	 * hack into WP $query object to allow displaying image posts with type "GalleryImage"
	 * 
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function _addImagePageCSS ($content)
	{
		global $post;
		$cssPath = $this->pluginUrl . 'css/gallery-image.css';
		echo '<link media="all" rel="stylesheet" type="text/css" href="' . $post->custom_css_href . '" />';
		//$precont .= '@import "' . $cssPath . '";';
	}
	/*
	 * find [myGallery/Gallery=id] embed codes and substitute those with real widget images specificaly for feeds
	 * 
	 * @param $content
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function _getGalleryInHTML ($datastr, $PS = false)
	{
		ob_start();
		global $wp_rewrite, $post;
		$json = new PSGalleryServicesJSON();
		$posturl = get_permalink($post->ID);
		$imageUrlSuf = ($wp_rewrite->permalink_structure ? 'image-page/' : (strstr($posturl, '?') ? '&' : '?') . 'image-page=');
		$datastr = stripcslashes(str_replace("Gallery = ", "", $datastr));
		$data = $json->decode($datastr);
		$data->title = rawurldecode($data->title);
		foreach ($data->ims as $key => $image)
		{
			//get first available thumb from the list defined on top (should be SQ)
			foreach ($this->thumbModes as $mode)
			{
				if ($image->thumbnails->{$mode})
				{
					$image->thumbnail = ($image->thumbnails->{$mode});
					break;
				}
			}
			$image->title = rawurldecode($image->title);
			$image->href = $posturl . $imageUrlSuf . ($key + 1);
			if ($PS)
			{
				$ImageUrl = PICTURESURF_URL_IMG . array_pop(explode('/', str_replace($mode . '_', '', $image->thumbnail)));
				$image->thumbnail = $ImageUrl;
				$image->href = $ImageUrl;
			}
			$data->ims[$key] == $image;
		}
		include ($this->includesDir . 'gallery_feed.php');
		return ob_get_clean();
	}
	/*
	 * utility function to transcript the version string into valid digit
	 * 
	 * @param $content
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function _versionNumber($str)
	{
		$num = 0;
		$parts = explode('.', $str);
		$dits = count($parts)-1;
		//any number in version can't go higher than 99 or it will overlap the higher number
		for($i = 0; $i < count($parts); $i++)
		{
			$num+= $parts[$i] * pow(10, ($dits-$i)*2);
		}
		return $num;
	}
	/*
	 * add galleries stats widget to the Dashboard
	 * 
	 * @param $content
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	function statsWidgetAdd()
	{
		if(!function_exists('wp_add_dashboard_widget'))
		{
			return;
		}
		$this->_initParams();
		if($this->params->Settings['IsImagePages'] != 1)
		{
			return;
		}
		$key = 'picturesurf-gallery';
		$title = 'How much does Picturesurf Gallery benefit me?';
		wp_add_dashboard_widget($key, $title, array(&$this, 'statsWidgetDisplay'));
		
		global $wp_meta_boxes;
		$normal = $wp_meta_boxes['dashboard']['normal']['core'];
		$backup = array($key => $normal[$key]);
		unset($normal[$key]);
		$sorted = array_merge($backup, $normal);
		$wp_meta_boxes['dashboard']['normal']['core'] = $sorted;
	}
	 */
	/*
	 * display the contents of the galleries stats widget
	 * 
	 * @param $content
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	function statsWidgetDisplay()
	{
		$this->_init();
		$row = $this->wpdb->get_row('SELECT SUM(views) as pageviews FROM ' . $this->wpdb->prefix . 'psimagepages;');
		$pageviews = $row->pageviews;
		$row = $this->wpdb->get_row('SELECT COUNT(id) as gals, SUM(views) as galviews FROM ' . $this->wpdb->prefix . 'psgalleries');
		$galviews = $row->galviews;
		$gals = $row->gals;
		$increase = ($galviews-$pageviews)==0?0:round(100*$pageviews/($galviews-$pageviews), 2);
		$pat = array('@image-page[\/=]@', '@http://@', '@\/@', '@\.@', '@\?@', '@\&@', '@=@', '@"@', '@\'@');
		$rep = array('',                  '',          '-S-',  '-D-',  '-Q-',  '-A-',  '-E-', '-d-', '-s-');
		$url = preg_replace($pat, $rep, $this->blogUrl);
		include ($this->includesDir . 'gallery_dashboard.php');
	}
	 */
}
?>